#ifndef __MYSQLCPPAPI_ALLOCATOR_H_
#define __MYSQLCPPAPI_ALLOCATOR_H_



#include "mysqlcppapi/exceptions/ex_base.h"

namespace mysqlcppapi
{


//Derive new Allocators to use e.g. ct_cmd_alloc and ct_cmd_drop instead of new and delete.
template<typename T_cobj>
class Allocator
{
public:
  typedef T_cobj cvalue_type;
  typedef T_cobj* cpointer;

  //An actual derived Allocator should implement these methods:
  //static cpointer allocate();
  //
  // ///Ignore do2ndStageDeallocation usually - see SharedPtr::set_do_2nd_stage_dellocation().
  //static void deallocate(cpointer pInstance, bool do2ndStageDeallocation);


};

//Allocator that implements allocate() and deallocate() with simple C++ new and delete.
template<typename T_cobj>
class Allocator_NewDelete : public Allocator<T_cobj>
{
public:  
  typedef typename Allocator<T_cobj>::cpointer cpointer;

  static cpointer allocate()
  {
    return new T_cobj();
  }
 
  static void deallocate(cpointer pInstance, bool /*do2ndStageDeallocation*/)
  {
    delete pInstance;
  }
};

//Like Allocator_NewDelete, but zeroes the struct memory:
template<typename T_cobj>
class Allocator_Zeroed : public Allocator_NewDelete<T_cobj>
{
public:
  typedef typename Allocator_NewDelete<T_cobj>::cpointer cpointer;
 
  static cpointer allocate()
  {
    //Call base class:
    cpointer pResult = Allocator_NewDelete<T_cobj>::allocate();
  
    //Zero the memory:
    memset(pResult, 0, sizeof(T_cobj));
    return pResult;
  }
};




} //namespace

#endif //__MYSQLCPPAPI_ALLOCATOR_H_

